% COORIENTATION_LOCAL  Compute the local co-orientation strength for two sets of localizations
%
% SYNOPSIS:
%  [K_par,K_ripley] = coorientation_local(localizations1,localizations2,size,zoomfactor,n_orientations,scale,rotations,R,f_patch,f_shift)
%
% PARAMETERS:
%   size
%      Size in x and y-direction of input localizations [CCD pixels]
%   zoomfactor
%      Pixels per unit distance in the positions list
%   n_orientations
%      Number of analyzed orientations (odd number)
%   scale
%      Scale at which the orientation is evaluated [CCD pixels]
%   rotations
%      Rotations of channel 2 for significance testing
%   R
%      Maximum distance of correlation scores used to compute co-orientation strength
%   f_patch
%      Size of the subregion of the image for which the co-orientation
%      strength is evaluated, as fraction of the full field of view
%   f_shift
%       Overlap between two adjacent subregions

%
% (C) Copyright 2015               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands
%
% Robert Nieuwenhuizen, April 2015

function varargout = coorientation_local(varargin)

d = struct('menu','Co-orientation',...
           'display','Paircorrelation weight',...
           'inparams',struct('name',       {'localizations1',    'localizations2',      'sz',                   'zm',           'n_orientations',           'scale',            'R',    'f_patch',          'f_shift'},...
                             'description',{'Localizations1',    'Localizations2',      'Size of input data',   'Zoomfactor',   'Number of orientations',   'Orientation scale','R',    'Rel. patch size',  'Overlap'},...
                             'type',       {'array',             'array',               'array',                'array',        'array',                    'array',            'array','array',            'array'},...
                             'dim_check',  {{[],[-1 -1]},        {[],[-1 -1]},          {[],0,[1 2],[2 1]},     {[],0},         0,                          0,                  0,      0,                  0},...
                             'range_check',{'R',                 'R',                   'N+',                   [eps Inf],      'N+',                       'R+',               'R+',   'R+',               'R+'},...
                             'required',   {0,                   0,                     0,                      0,              1,                          0,                  0,      1,                  1},...
                             'default',    {[],                  [],                    [],                     1,              41,                         2,                  2,      0.1,                0.5}...
                              ),...
           'outparams',struct('name',{'Pval','K_par','c_ringav','K_ripley'},...
                              'description',{'P-value','Co-orientation strength vs rotation','Correlation score vs distance and angle','Ripleys K'},...
                              'type',{'array','array','image','array'}...
                              )...
           );       
      
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end

try
   [localizations1,localizations2,sz,zm,n_orientations,ornt_scale,R,f_patch,f_shift] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end  

% Fixed parameter
ornt_bandwidth = 0.8;   % Bandwidth of the orientation space filters

%% Estimate orientations in both channels
im1 = showlocalizations(localizations1,sz,zm);
im1_orientations = filamentorientation_multi(im1,n_orientations,ornt_scale*zm,ornt_bandwidth);
im2 = showlocalizations(localizations2,sz,zm);
im2_orientations = filamentorientation_multi(im2,n_orientations,ornt_scale*zm,ornt_bandwidth);

sz_full = imsize(im1_orientations);

%% Define grid for local computation
sz_patch = round(sz_full(1:2)*f_patch);
n_samples = (1/f_patch-1)*1/f_shift+1;
st_x = round(linspace(1,sz_full(1)-sz_patch(1)+1,n_samples+1));
st_x = st_x - 1;
ed_x = st_x + (sz_patch(1)-1);

st_y = round(linspace(1,sz_full(2)-sz_patch(2)+1,n_samples+1));
st_y = st_y - 1;
ed_y = st_y + (sz_patch(2)-1);


%% Initialize variables

% Image sizes 
sz_ext = [2*sz_patch n_orientations];   % Size of extended orienation space images of patches
sz_cut = [sz_patch,n_orientations];     % Size for cutting cross-correlation

% Relative angles
dPhi = linspace(-pi/2,pi/2,n_orientations+1);
dPhi = (dPhi(1:end-1)+dPhi(2:end))/2;
dPhi = permute(dPhi,[1 3 2]);

% Compute weight image for cross-correlation
W_im = paircorrelation_weight(localizations1,localizations2,sz_patch,1);
W_im = W_im*(f_patch^4);

% Mask for co-localization part of cross-correlation
mask_R = rr(sz_patch)<(R*zm);

%% Compute co-orientation strength per patch
K_par_store = zeros(length(st_x),length(st_y));
K_ripley_store = zeros(length(st_x),length(st_y));

if exist('parfor','builtin')
    PAR_EVAL = 1;
    if exist('parpool','builtin')
        
        parpool;
    elseif exist('matlabpool','builtin')
        par_scheduler = findResource('scheduler','type','local');
        if par_scheduler.ClusterSize > 1
            matlabpool(par_scheduler.ClusterSize-1)
        end
        matlabpool;
    else
        PAR_EVAL = 0;
    end
else
    PAR_EVAL = 0;
end

for ii = 1:length(st_x)
    % Preallocate selections
    sel1 = cell(length(st_y),1);
    sel2 = cell(length(st_y),1);
    for jj = 1:length(st_y)
        sel1{jj} = im1_orientations(st_x(ii):ed_x(ii),st_y(jj):ed_y(jj),:);
        sel2{jj} = im2_orientations(st_x(ii):ed_x(ii),st_y(jj):ed_y(jj),:);
    end    
    
    if PAR_EVAL
        parfor jj = 1:length(st_y)    
            im1_sel = sel1{jj};
            im2_sel = sel2{jj};
            im1_sel = extend(im1_sel,sz_ext,'symmetric',0);
            im2_sel = extend(im2_sel,sz_ext,'symmetric',0);

            % Compute co-orientation
            im1_sel = ft(im1_sel);
            im2_sel = conj(ft(im2_sel));
            c = im1_sel*im2_sel;
            c = sqrt(prod(imsize(im1_sel)))*real(ft(c));
            c = cut(c,sz_cut);        
            c = c/W_im/(1/n_orientations);
            K_par = sum(c*mask_R*2*cos(2*dPhi))/sum(mask_R)/n_orientations;
            K_par_store(ii,jj) = K_par;

            K_ripley = sum(c*mask_R)/sum(mask_R)/n_orientations;
            K_ripley_store(ii,jj) = K_ripley;
        end
    
    else
        for jj = 1:length(st_y)   
            im1_sel = sel1{jj};
            im2_sel = sel2{jj};
            im1_sel = extend(im1_sel,sz_ext,'symmetric',0);
            im2_sel = extend(im2_sel,sz_ext,'symmetric',0);

            % Compute co-orientation
            im1_sel = ft(im1_sel);
            im2_sel = conj(ft(im2_sel));
            c = im1_sel*im2_sel;
            c = sqrt(prod(imsize(im1_sel)))*real(ft(c));
            c = cut(c,sz_cut);        
            c = c/W_im/(1/n_orientations);
            K_par = sum(c*mask_R*2*cos(2*dPhi))/sum(mask_R)/n_orientations;
            K_par_store(ii,jj) = K_par;

            K_ripley = sum(c*mask_R)/sum(mask_R)/n_orientations;
            K_ripley_store(ii,jj) = K_ripley;
        end
    end
    
end

%% Compute co-orientation map
[X_patch,Y_patch] = meshgrid((st_x+ed_x)'/2,(st_y+ed_y)'/2);
[X_pix,Y_pix] = meshgrid((0:sz_full(1)-1),(0:sz_full(2)-1));

% Interpolate co-orientation strength
F = scatteredInterpolant(X_patch(:),Y_patch(:),K_par_store(:));
K_par_im = F(X_pix,Y_pix);
K_par_im = K_par_im';

if nargout>1
    % Interpolate Ripley's K
    F = scatteredInterpolant(X_patch(:),Y_patch(:),K_ripley_store(:));
    K_ripley_im = F(X_pix,Y_pix);
    K_ripley_im = K_ripley_im';
    
    varargout{1} = mat2im(K_par_im);
    varargout{2} = mat2im(K_ripley_im);
    
else
    varargout{1} = mat2im(K_par_im);
end