% COORIENTATION_PLOT   Compute the coorientation plot for two sets of localizations
%
% SYNOPSIS:
%  [c_full,c_ringav] = coorientation_plot(localizations1,localizations2,size,zoomfactor,n_orientations,scale)
%
% PARAMETERS:
%   size
%      Size in x and y-direction of input localizations [CCD pixels]
%   zoomfactor
%      Pixels per unit distance in the positions list
%   n_orientations
%      Number of analyzed orientations (odd number)
%   scale
%      Scale at which the orientation is evaluated [CCD pixels]

% (C) Copyright 2015               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands
%
% Robert Nieuwenhuizen, April 2015
%
function varargout = coorientation_plot(varargin)

d = struct('menu','Co-orientation',...
           'display','Paircorrelation weight',...
           'inparams',struct('name',       {'localizations1',    'localizations2',      'sz',                   'zm',           'n_orientations',           'scale'},...
                             'description',{'Localizations1',    'Localizations2',      'Size of input data',   'Zoomfactor',   'Number of orientations',   'Orientation scale'},...
                             'type',       {'array',             'array',               'array',                'array',        'array',                    'array'},...
                             'dim_check',  {{[],[-1 -1]},        {[],[-1 -1]},          {[],0,[1 2],[2 1]},     {[],0},         0,                          0},...
                             'range_check',{'R',                 'R',                   'N+',                   [eps Inf],      'N+',                       'R+'},...
                             'required',   {0,                   0,                     0,                      0,              1,                          0},...
                             'default',    {[],                  [],                    [],                     1,              41,                         2}...
                              ),...
           'outparams',struct('name',{'c_score','c_ringav'},...
                              'description',{'Score vs shift and angle','Score vs distance and angle'},...
                              'type',{'image','image'}...
                              )...
           );       

if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end

try
   [localizations1,localizations2,sz,zm,n_orientations,ornt_scale] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end     

% Fixed parameter
ornt_bandwidth = 0.8;   % Bandwidth of the orientation space filters

% Compute weight image for cross-correlation 
%   The last argument forces a circular mask over the ROI, since this is
%   also used for significance testing. This can be disabled though.
[W_im,W_mask,M] = paircorrelation_weight(localizations1,localizations2,sz,zm,'circle');

% Estimate orientations in channel 1
im1 = showlocalizations(localizations1,sz,zm);
im1_orientations = filamentorientation_multi(im1,n_orientations,ornt_scale*zm,ornt_bandwidth);
sz_tmp = imsize(im1_orientations);
im1_orientations = extend(im1_orientations*M,[2*sz_tmp(1:2) sz_tmp(3)],'symmetric',0);

% Estimate orientations in channel 1
im2 = showlocalizations(localizations2,sz,zm);
c_full = filamentorientation_multi(im2,n_orientations,ornt_scale*zm,ornt_bandwidth);
c_full = extend(c_full*M,[2*sz_tmp(1:2) sz_tmp(3)],'symmetric',0);

% Compute cross-correlation in space and across orientations
im1_orientations = ft(im1_orientations);
c_full = conj(ft(c_full));
c_full = im1_orientations*c_full;
c_full = sqrt(prod(imsize(im1_orientations)))*real(ft(c_full));
c_full = cut(c_full,sz_tmp);
c_full = c_full/W_im;
c_full = c_full/(1/n_orientations);

% Average cross-correlation score over rings of constant spatial distance
if nargout > 1
   r_steps = floor(min(imsize(c_full,1),imsize(c_full,2))/2)+1;
   c_ringav = newim(r_steps,n_orientations);
   for nn = 1:n_orientations
       c_ringav(:,nn-1) = radialmean(c_full(:,:,nn-1),[],1,1);
   end
end

%% Outputs
varargout{1} = c_full;      % Correlation score vs x,y,angle
varargout{2} = c_ringav;    % Correlation score vs r,angle
