% COORIENTATION_SIGTEST  Test the statistical significance of the co-orientation for two sets of localizations
%
% SYNOPSIS:
%  [c_full,c_ringav] = coorientation_sigtest(localizations1,localizations2,size,zoomfactor,n_orientations,scale,rotations,R)
%
% PARAMETERS:
%   size
%      Size in x and y-direction of input localizations [CCD pixels]
%   zoomfactor
%      Pixels per unit distance in the positions list
%   n_orientations
%      Number of analyzed orientations (odd number)
%   scale
%      Scale at which the orientation is evaluated [CCD pixels]
%   rotations
%      Rotations of channel 2 for significance testing
%   R
%      Maximum distance of correlation scores used to compute co-orientation strength

%
% (C) Copyright 2015               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands
%
% Robert Nieuwenhuizen, April 2015
%
function varargout = coorientation_sigtest(varargin)

d = struct('menu','Co-orientation',...
           'display','Paircorrelation weight',...
           'inparams',struct('name',       {'localizations1',    'localizations2',      'sz',                   'zm',           'n_orientations',           'scale',            'K_rotations',       'R'},...
                             'description',{'Localizations1',    'Localizations2',      'Size of input data',   'Zoomfactor',   'Number of orientations',   'Orientation scale','Rotations',         'R'},...
                             'type',       {'array',             'array',               'array',                'array',        'array',                    'array',            'array',             'array'},...
                             'dim_check',  {{[],[-1 -1]},        {[],[-1 -1]},          {[],0,[1 2],[2 1]},     {[],0},         0,                          0,                  0,                   0},...
                             'range_check',{'R',                 'R',                   'N+',                   [eps Inf],      'N+',                       'R+',               'N+',                'R+'},...
                             'required',   {0,                   0,                     0,                      0,              1,                          0,                  0,                   0},...
                             'default',    {[],                  [],                    [],                     1,              41,                         2,                  20,                  2}...
                              ),...
           'outparams',struct('name',{'Pval','K_par','c_ringav','K_ripley'},...
                              'description',{'P-value','Co-orientation strength vs rotation','Correlation score vs distance and angle','Ripleys K'},...
                              'type',{'array','array','image','array'}...
                              )...
           );       
      
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end

try
   [localizations1,localizations2,sz,zm,n_orientations,ornt_scale,K_rotations,R] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end     

%% Initialize variables

% Fixed parameter
ornt_bandwidth = 0.8;   % Bandwidth of the orientation space filters

% Compute weight image for cross-correlation 
%   The last argument forces a circular mask over the ROI, since this is
%   also used for significance testing. This can be disabled though.
[W_im,W_mask,M] = paircorrelation_weight(localizations1,localizations2,sz,zm,'circle');

% Relative angles
dPhi = linspace(-pi/2,pi/2,n_orientations+1);
dPhi = (dPhi(1:end-1)+dPhi(2:end))/2;
dPhi = permute(dPhi,[1 3 2]);

% Mask for co-localization part of cross-correlation
mask_R = rr(sz*zm)<(R*zm);

% Rotation matrix
dTheta = 2*pi/K_rotations;
rot_mat = [cos(dTheta) -sin(dTheta); sin(dTheta) cos(dTheta)];

%% Estimate orientations in channel 1
im1 = showlocalizations(localizations1,sz,zm);
im1_orientations = filamentorientation_multi(im1,n_orientations,ornt_scale*zm,ornt_bandwidth);
sz_tmp = imsize(im1_orientations);
im1_orientations = extend(im1_orientations*M,[2*sz_tmp(1:2) sz_tmp(3)],'symmetric',0);


%% Compute co-orientation strenght vs channel 2 rotation
K_par = zeros(K_rotations,1);
K_ripley = zeros(K_rotations,1);

fprintf('Co-orientation significance test\n')
for aa = 1:K_rotations
    fprintf('Rotation %d of %d\n',aa,K_rotations)
    % Bin image 2
    im2 = showlocalizations(localizations2,sz,zm);
    c_full = filamentorientation_multi(im2,n_orientations,ornt_scale*zm,ornt_bandwidth);
    c_full = extend(c_full*M,[2*sz_tmp(1:2) sz_tmp(3)],'symmetric',0);
    
    % Compute cross-correlation
    if aa == 1
        im1_orientations = ft(im1_orientations);
    end
    c_full = conj(ft(c_full));
    c_full = im1_orientations*c_full;
    c_full = sqrt(prod(imsize(im1_orientations)))*real(ft(c_full));
    c_full = cut(c_full,sz_tmp);
    c_full = c_full/W_im;
    c_full = c_full/(1/n_orientations);
    
    % Average cross-correlation score over rings of constant spatial distance
    if aa == 1
       r_steps = floor(min(imsize(c_full,1),imsize(c_full,2))/2)+1;
       c_ringav = newim(r_steps,n_orientations);
       for nn = 1:n_orientations
           c_ringav(:,nn-1) = radialmean(c_full(:,:,nn-1),[],1,1);
       end
    end
    
    % Compute co-orientation strength and Ripley's K
    K_par(aa) = sum(c_full*mask_R*2*cos(2*dPhi))/sum(mask_R)/n_orientations;
    K_ripley(aa) = sum(c_full*mask_R)/sum(mask_R)/n_orientations;    

    clear c_full
    
    % Rotate the localizations in channel 2
    localizations2(:,1:2) = localizations2(:,1:2)+0.5-repmat(sz/2,[size(localizations2,1) 1]);
    localizations2(:,1:2) = localizations2(:,1:2)*rot_mat;
    localizations2(:,1:2) = localizations2(:,1:2)-0.5+repmat(sz/2,[size(localizations2,1) 1]);

end

%% Significance test
   
% Significance test
Kf = fft(K_par);
sigK = sqrt(sum((K_par)/K_rotations).^2+2*sum(imag(Kf).^2)/(length(Kf)^2));
Pval = normcdf(-K_par(1),0,sigK);

fprintf('R\t\t: %3.2f CCD pixels\n',R);
fprintf('P-value\t: %3.2d\n',Pval)
if Pval>0.01
    fprintf('No significant co-orientation at 0.01 level.\n');
else
    fprintf('Significant co-orientation at 0.01 level.\n');
end

%% Outputs
varargout{1} = Pval;
varargout{2} = K_par;
varargout{3} = c_ringav;
varargout{4} = K_ripley;
