% PAIRCORRELATION_WEIGHT   Compute mask and weight for the pair-correlation function
%
% SYNOPSIS:
%  [correlation_weight,correlation_mask,image_mask,w_ring,n_ring] = paircorrelation_weight(localizations1,localizations2,size,zoomfactor,roi_method,roi_scale)
%
% PARAMETERS:
%   size
%      Size in x and y-direction of input localizations [CCD pixels]
%   zoomfactor
%      Pixels per unit distance in the positions list
%   roi_method
%      Method for determining ROI: full (default), circle, selection, segmentation
%   roi_scale
%      Smoothing scale for automatic ROI determination
%
% (C) Copyright 2015               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands
% Robert Nieuwenhuizen, April 2015
%
function varargout = paircorrelation_weight(varargin)

d = struct('menu','FRC resolution',...
           'display','Paircorrelation weight',...
           'inparams',struct('name',       {'localizations1',    'localizations2',      'sz',                   'zm',       'roi_method',                                   'roi_scale'},...
                             'description',{'Localizations1',    'Localizations2',      'Size of input data',   'Zoom',     'Auto ROI',                                     'Scale'},...
                             'type',       {'array',             'array',               'array',                'array',    'option',                                       'array'},...
                             'dim_check',  {{[],[-1 -1]},        {[],[-1 -1]},          {[],0,[1 2],[2 1]},     {[],0},     0,                                              0},...
                             'range_check',{'R',                 'R',                   'N+',                   [eps Inf],  {'full','circle','selection','segmentation'},   'N+'},...
                             'required',   {0,                   0,                     0,                      0,          0,                                              0},...
                             'default',    {[],                  [],                    [],                     1,          'full',                                         1}...
                              ),...
           'outparams',struct('name',{'correlation_weight','correlation_mask','image_mask','w_ring','n_ring'},...
                              'description',{'Correlation weight','Correlation mask','Image mask','Weight per ring','Pixels per ring'},...
                              'type',{'image','image','image','array','array'}...
                              )...
           );       

if nargin == 1    
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = struct('menu','none');
      return
   end
end

try
   [localizations1,localizations2,sz,zm,roi_method,roi_scale] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end       

if roi_scale<=1
    roi_scale = zm;
end

% ROI selection
switch roi_method
    case 'full'
        M = ones(sz*zm);   
        M = dip_image(M,'single');
        
    case 'circle'
        % Visualize both channels
        im1 = showlocalizations(localizations1,sz,zm);
        im2 = showlocalizations(localizations2,sz,zm);
        
        M = rr(sz*zm);
        M = (M<(min(sz)*zm/2));
        
    case 'segmentation'
        % Visualize both channels
        im1 = showlocalizations(localizations1,sz,zm);
        im2 = showlocalizations(localizations2,sz,zm);
        
        M = threshold(gaussf(im1,roi_scale),'background',1)|threshold(gaussf(im2,roi_scale),'background',1);
        M = fillholes(M);
        
    case 'selection'
        % Visualize both channels
        im1 = showlocalizations(localizations1,sz,zm);
        im2 = showlocalizations(localizations2,sz,zm);
        
        h = dipshow(joinchannels('rgb',im1,im2),[0 1]);
        M = diproi(h);
        close(h)
end

if strcmp(roi_method,'full')
    W_sum = size(localizations1,1)*size(localizations2,1)/(prod(imsize(M))^2);
else
    W_sum = (sum(im1*M)*sum(im2*M)/sum(M)^2);       % Average channel intensities in ROI
end

% Compute paircorrelation weight
sz = sz*zm;
W_im = extend(M,2*sz(:,1:2),'symmetric',0);     % Zero padding to avoid periodic boundaries
W_im = dip_image(W_im,'single');
W_im = crosscorrelation(W_im,W_im);
W_im = W_im*sqrt(prod(imsize(W_im)));
W_im = cut(W_im,sz);                            % Recut weight image to original image size
W_thres = W_im>1;                               % Threshold pixels with no overlap
W_im(~W_thres) = 1;
W_im = W_im*W_sum;

% Compute number of pixels per correlating ring
n_ring = radialsum(W_thres);
n_ring = im2mat(n_ring);
w_ring = radialsum(W_im);
w_ring = im2mat(w_ring)./n_ring;

% Generate outputs
varargout{1} = W_im;
varargout{2} = W_thres;
varargout{3} = M;
varargout{4} = w_ring;
varargout{5} = n_ring;


