% (C) Copyright 2015               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands
%
% Robert Nieuwenhuizen, April 2015
%
% This code requires the toolbox DIPimage to be installed
% (http://www.diplib.org/download)
%
% for example purposes we reduced the computation complexity compare to the
% evaluation in the manuscript. Less orientation, larger super-resolution
% pixels and less rotations

close all
clearvars

%% Parameters for analysis
ornt_scale = 2;         % Scale at which the orientation is assessed [CCD pixels]
n_orientations = 21;    % Angular bin size

sz_ccd = 180*[1 1];     % Field of view size [CCD pixels]
pixelsize_ccd = 100;    % CCD pixel size [nm]
zm = 5;                 % Superresolution pixel size / CCD pixel size

% Significance testing
K_rotations = 10;        % Rotations for significance test
R = 5;                  % Maximum distance for computing co-orientation strength [CCD pixels]

% Local co-orientation strength
d_patch =  2e3;                                 % Patch size [nm]
f_shift = 1/2;                                  % Patch overlap

%% Load data
load('testdata_HUVEC_Tub488Vim647.mat','coords_tub','coords_vim')

%% Load color map
load('cmap_fire.mat')

%% Show localizations
im_tub = showlocalizations(coords_tub,sz_ccd,zm,'jitter');
im_vim = showlocalizations(coords_vim,sz_ccd,zm,'jitter');

h1 = dipshow(joinchannels('rgb',im_tub,im_vim),[0 3]);

%% Select ROI
fprintf('Please select a rectangular ROI from the image with the left mouse button.\n');
[~,roi_sel] = dipcrop(h1);
hold on
roi_sel(2) = min(roi_sel([2 4]));
roi_sel(4) = roi_sel(2);
rectangle('Position',roi_sel([1 3 2 4]),'LineWidth',2,'EdgeColor','w','Curvature',[1 1])
hold off

fprintf('Computing ...\n')
% Create a mask of localizations that are to be removed
mask_roi = im_vim; 
mask_roi(:,:) = 0;
mask_roi(roi_sel(1,1):roi_sel(1,1)+roi_sel(2,1),roi_sel(1,2):roi_sel(1,2)+roi_sel(2,2)) = 1;

coords_tub_sel = masklocalizations(coords_tub,mask_roi,zm);
coords_vim_sel = masklocalizations(coords_vim,mask_roi,zm);
clear mask_roi

% Scale ROI coordinates
roi_sel(2,:) = roi_sel(2,:) + 1;
roi_sel = roi_sel / zm;
roi_sel(1,:) = roi_sel(1,:) - 0.5;

% Define topleft pixel of ROI as (x,y) = (0,0)
coords_vim_sel(:,1) = coords_vim_sel(:,1) - roi_sel(1,1);
coords_vim_sel(:,2) = coords_vim_sel(:,2) - roi_sel(1,2);
coords_tub_sel(:,1) = coords_tub_sel(:,1) - roi_sel(1,1);
coords_tub_sel(:,2) = coords_tub_sel(:,2) - roi_sel(1,2);

sz_sel = floor(roi_sel(2,:));

%% Compute co-orienation plot

% Coorientation score
[~,c_ringav] = coorientation_plot(coords_vim_sel,coords_tub_sel,sz_sel,zm,n_orientations,ornt_scale);

% Angle differences for which the coorientation plot is obtained
dPhi = linspace(-pi/2,pi/2,n_orientations+1);
dPhi = (dPhi(1:end-1)+dPhi(2:end))/2;
%dPhi = permute(dPhi,[1 3 2]);

% Distances for which the coorientation plot is obtained
r = ((1:size(c_ringav,1))-1)*pixelsize_ccd/zm;
% Plot

figure
imagesc(squeeze(dPhi),r,im2mat(c_ringav)');
xlabel('Relative angle (rad)')
ylabel('Distance (nm)')
% title(sprintf('Orientation scale: %3.0f nm',ornt_scale*pixelsize_ccd))
fprintf('Orientation scale: %3.0f nm.\n',ornt_scale*pixelsize_ccd);
caxis([0.75 2])
ylim([0 2e3])
colorbar('LineWidth',1)
colormap(cmap)
set(gca,'XTick',-1.5:0.5:1.5,'YDir','normal')

%% Significance test
[Pval,K_par,c_ringav,K_ripley] = coorientation_sigtest(coords_vim_sel,coords_tub_sel,sz_sel,zm,n_orientations,ornt_scale,K_rotations,R);

% Plot result
Theta = linspace(0,2*pi,K_rotations+1); Theta = Theta(1:end-1);
Theta(Theta>pi) = Theta(Theta>pi)-2*pi;
Theta = circshift(Theta,round(K_rotations/2)-1,2);
Kf = fft(K_par);
sigK = sqrt(sum((K_par)/K_rotations).^2+2*sum(imag(Kf).^2)/(length(Kf)^2));
clear Kf

figure
hold on
plot(Theta,circshift(K_par,round(K_rotations/2)-1),'LineWidth',1)
plot([-pi pi],[0 0],'k','LineWidth',1)
set(gca,'YLim',get(gca,'YLim'))
plot([0 0],get(gca,'YLim'),'k','LineWidth',1)
plot([-pi pi],norminv(0.99,0,sigK)*[1 1],'k:','LineWidth',1)
hold off
xlabel('\theta','FontName','Arial')
ylabel('K_{||}(R;\theta)')
set(gca,'Box','on','LineWidth',1)
xlim([-pi pi])

%% Local co-orientation strength
f_patch = (d_patch/pixelsize_ccd)/min(sz_ccd);  % Patch size / FOV size
K_local = coorientation_local(coords_vim,coords_tub,sz_ccd,zm,n_orientations,ornt_scale,R,f_patch,f_shift);

% Compute image clipping values
c_quantile = 0.98;                                                  % Fraction of non-zero pixels that are not clipped
c1_vals = im2mat(im_tub(:)); c1_vals = c1_vals(c1_vals>1E-3);
c2_vals = im2mat(im_vim(:)); c2_vals = c2_vals(c2_vals>1E-3);
c1_max = quantile(c1_vals,c_quantile);
c2_max = quantile(c2_vals,c_quantile);

% Stretch images
im_tub(im_tub>c1_max) = c1_max;
im_vim(im_vim>c2_max) = c2_max;
im_tub = stretch(im_tub,0,100,0,255);
im_vim = stretch(im_vim,0,100,0,255);
K_overlay = stretch(K_local*(K_local>0),0,97,0,255);

dipshow(joinchannels('rgb',im_tub,im_vim,K_overlay),[0 255])


